//*****************************************************************************
//*
//*
//*		SynchFile.cpp
//*
//*
//*****************************************************************************
#include	"AZ_Lib.h"
#include	"SynchFile.h"
#include 	<winbase.h>


int	      (*synch_name   )(void *,const char*,const char*)=0;
int		  (*synch_percent)(void *,int)=0;
void	   *synch_param =0;
const char *synch_source=0;
const char *synch_target=0;
HWND		synch_hwnd  =0;
BOOL		synch_cancel=0;


static	BOOL			synch_isstart=0;
static	CFileFinder		suche1,suche2;

//*****************************************************************************
//*
//*		SynchStop
//*
//*****************************************************************************
int SynchStop()
{

	if(!synch_isstart)return 0;

	suche1.Break();
	suche2.Break();
	synch_cancel=1;

return 1;
}

//*****************************************************************************
//*
//*		SynchWriteTime
//*
//*****************************************************************************
//	Liest die Zeit der letzten nderung von File aus
//	Ergibt 0 bei einem Fehler ansonsten die Zeit
__int64 SynchWriteTime(const char *File)
{
WIN32_FILE_ATTRIBUTE_DATA	find;
__int64						zeit;


	if(!GetFileAttributesEx(File,GetFileExInfoStandard,&find))
		{
		return 0;
		}

	zeit  = *(__int64*)(&find.ftLastWriteTime);
	zeit -= 0x019DB1DED53E8000;
	zeit /= 10000000*2;						// 100ns zu Sekunden umwandeln


return zeit;
}

//*****************************************************************************
//*
//*      SynchCopyFile
//*
//*****************************************************************************
//	Kopiert die Datei Quelle nach Ziel
//	Schreibgeschtzte Dateien werden berschriben
//	Auch der Dateipfad wird erstellt wenn ntig
//	Ergibt 1 wenn die Datei kopiert wurde.
int SynchCopyFile(const char *Quelle,const char *Ziel)
{
char		mem[MAX_PATH];
int			x;





		x=CopyFile(Quelle,Ziel,FALSE);			// Datei kopieren

	if(!x && GetLastError()==ERROR_ACCESS_DENIED)
		{
		RemoveFile(Ziel);
		x=CopyFile(Quelle,Ziel,FALSE);
		}

	if(!x && GetLastError()==ERROR_PATH_NOT_FOUND)
		{
		GetFilePath (mem,Ziel,sizeof(mem));
		DelBackSlash(mem);
		CreatePath  (mem);						// Pfad erzeugen
		x=CopyFile(Quelle,Ziel,FALSE);
		}


	if(x)										// Kopiere Zeitdaten
		{
		CopyFileTime(Quelle,Ziel) ;
		}



return x;
}

//*****************************************************************************
//*
//*      SynchCopy
//*
//*****************************************************************************
// 	Synchonisiert eine Serie von Dateien
//	Pfad1	: Erster  Pfad oder Datei die synchronisiert werden sollen
//	Pfad2	: Zweiter Pfad oder Datei die synchronisiert werden sollen
//	Plus	: Dateifilter fr Dateien die synchronisiert werden sollen
//			  z.B.:   "*.bmp;*.gif;*jpg"
//	Minus	: Dateifilter fr Dateien die nicht synchronisiert werden sollen
//			  z.B.:   "Auto*.bmp"
//	Art		: Ist Bit 0!=0 werden nur neuere Dateien kopiert
//			  Ist Bit 1!=0 werden Fehlermeldungen ausgegeben
//			  Ist Bit 2!=0 werden auch Unterverszeichnisse synchronisiert
// 			  Ist Bit 3!=0 wird eine vorhandene Datei nie berschrieben
//			  Ist Bit 4!=0 wird in beiden Richtungen synchonisiert
//				               (auch von Pfad2 nach Pfad1)
//			  Ist Bit 5!=0 werden schreibgeschtzte Dateien nicht berschrieben
//			  Ist Bit 6!=0 werden berzhlige Dateien entfernt
//			  Ist Bit 7!=0 wird bei jeder Dateigefrag ob sie kopiert werden soll
//	Ergibt   >=0 die Anzahl der kopierten Dateien
//			  -1 bei einem Fehler
//			  -2 bei einem Fehler mit Abbruch
int SynchCopy(	const char *Pfad1,const char *Pfad2,const char *Plus,const char *Minus,int Art)
{
__int64			 zeit1,zeit2,diff;
CFileFinder		*suche;
char			*name,puffer[MAX_PATH],mem[MAX_PATH*3];
int				 x,y,ok,attr,pos,error=0,snr=0,kopie_anz=0,stop=0;
__int64			 len_max=0,len_pos=0,prozent_next=3;



	if(synch_isstart    )return -1;
	if(!Pfad1 || !*Pfad1)return -1;
	if(!Pfad2 || !*Pfad2)return -1;

	len_max=0;
	len_pos=0;

	synch_cancel=0;
	synch_isstart=1;

	//******************** Einzelne Datei Synchronisieren *********************

	if( Art&0x40)Art&=~0x19;
	if( Art&0x08)Art&=~0x20;
	if(~Art&0x01)Art&=~0x10;

	x=ExistFile(Pfad1);
	y=ExistFile(Pfad2);
	if(x==1 && y >1){synch_isstart=0;return -1;}
	if(x> 1 && y==1){synch_isstart=0;return -1;}

	if(x==1 && y==1)								// Zwei Dateien ?
		{
		if(Art&0x40)								// berzhlige Dateien lschen
			{
			synch_isstart=0;
			return 0;
			}

		if(Art&0x20)								// Schreibgeschtze ignoriren
			{
			attr=GetFileAttributes(Pfad2);
			if(attr>0 && attr&FILE_ATTRIBUTE_READONLY)
				{
				synch_isstart=0;
				return 0;
				}
			}

		zeit1=SynchWriteTime(Pfad1);
		zeit2=SynchWriteTime(Pfad2);

		if(zeit1 && zeit2)							// Welche Datei ist jnger
			{
			if(Art&0x08)							// Nie berschreiben
				{
				synch_isstart=0;
				return 0;
				}

			if(Art&0x01)							// Nur neuere berschreiben
				{
				diff = zeit2-zeit1;
													// Sommerzeitfehler bei FAT/NTFS
				if(diff>= 1799 && diff<= 1801)zeit2=zeit1;
				if(diff<=-1799 && diff>=-1801)zeit2=zeit1;

				if(!(Art&0x10) && zeit1<zeit2+1)
					{
					synch_isstart=0;
					return 0;
					}
				if(zeit1< zeit2+1)x=0;
				if(zeit1> zeit2-1)y=0;
				if(x==0 && y==0){synch_isstart=0;return 0;}
				}
			else{
				if(Art&0x10 && zeit1<zeit2+1)
						x=0;
				else	y=0;
				}
			}
		else{
			if(zeit1==0)x=0;
			if(zeit2==0)y=0;
			}
		}
	
	puffer[0] = 0;
	mem   [0] = 0;

	if(synch_name)									// Dateinamen anzeigen
		{
		synch_source=(x==0 && y==1)? Pfad2:Pfad1;
		synch_target=(x==0 && y==1)? Pfad1:Pfad2;
		   stop=synch_name(synch_param,synch_source,synch_target);
		if(stop){synch_isstart=0;return -2;}
		}

	if(x==1 && y==0)								// Einzelne Datei kopieren
		{
		if(Art&0x40)								// berzhlige Dateien lschen
			{
			synch_isstart=0;
			return 0;
			}

		if(Art&0x80)								// Abfrage
			{
			strcpy (mem,"Soll\n");
			strlcat(mem,Pfad1,MAX_PATH);
			strlcat(mem,"\nnach\n",MAX_PATH);
			strlcat(mem,Pfad2,MAX_PATH);
			strlcat(mem,"\nkopiert werden ?",MAX_PATH);

			   ok=MessageBox(synch_hwnd,mem,"Frage",MB_ICONQUESTION|MB_YESNOCANCEL);
			if(ok==IDCANCEL){synch_isstart=0;return -2;}
			if(ok==IDNO    ){synch_isstart=0;return  0;}
			}

		   x=SynchCopyFile(Pfad1,Pfad2);
		if(x)kopie_anz++;

		if(synch_name)								// Dateinamen anzeigen
			{
			synch_name(synch_param,Pfad1,Pfad2);
			}

		if(synch_percent)							// Prozentanzeige
			{
			synch_percent(synch_param,100);
			}

		if(synch_cancel)							// Benutzer Abbruch
			{
			synch_isstart=0;
			return -2;
			}

		if(!x && (Art&0x02))						// Fehlertext ausgeben
			{
			strcpy (mem,"Fehler beim Kopieren von:\n\n");
			strlcat(mem,Pfad1,MAX_PATH);

			   ok=MessageBox(synch_hwnd,mem,0,MB_ICONERROR|MB_OKCANCEL);
			if(ok==IDCANCEL)return -2;
			}

		synch_isstart=0;
		return (x)? kopie_anz:-1;
		}

	if(x==0 && y==1)								// Einzelne Datei kopieren
		{
		if(Art&0x40)								// berzhlige Dateien lschen
			{
			if(Art&0x20)							// Schreibgeschtze ignoriren
				{
				attr=GetFileAttributes(Pfad2);
				if(attr>0 && attr&FILE_ATTRIBUTE_READONLY)
					{
					synch_isstart=0;
					return 0;
					}
				}

			if(Art&0x40)							// Abfrage
				{
				strcpy (mem,"Soll die Datei\n");
				strlcat(mem,Pfad2,MAX_PATH);
				strlcat(mem,"\ngelscht werden ?",MAX_PATH);

				   ok=MessageBox(synch_hwnd,mem,"Frage",MB_ICONQUESTION|MB_YESNOCANCEL);
				if(ok==IDCANCEL){synch_isstart=0;return -2;}
				if(ok==IDNO    ){synch_isstart=0;return  0;}
				}

			x=RemoveFile(Pfad2);
			}
		else{
			if(~Art&0x10)							// Nur in eine Richtung
				{
				synch_isstart=0;
				return 0;
				}

			if(Art&0x80)							// Abfrage
				{
				strcpy (mem,"Soll\n");
				strlcat(mem,Pfad2,MAX_PATH);
				strlcat(mem,"\nnach\n",MAX_PATH);
				strlcat(mem,Pfad1,MAX_PATH);
				strlcat(mem,"\nkopiert werden ?",MAX_PATH);

				   ok=MessageBox(synch_hwnd,mem,"Frage",MB_ICONQUESTION|MB_YESNOCANCEL);
				if(ok==IDCANCEL){synch_isstart=0;return -2;}
				if(ok==IDNO    ){synch_isstart=0;return  0;}
				}

			x=SynchCopyFile(Pfad2,Pfad1);
			}

		if(x)kopie_anz++;

		if(synch_name)								// Dateinamen anzeigen
			{
			synch_name(synch_param,Pfad2,Pfad1);
			}

		if(synch_percent)							// Prozentanzeige
			{
			synch_percent(synch_param,100);
			}

		if(synch_cancel)							// Benutzer Abbruch
			{
			synch_isstart=0;
			return -2;
			}

		if(!x && (Art&0x02))						// Fehlertext ausgeben
			{
			strcpy (mem,(Art&0x40)? "Fehler beim Lschen von:\n\n":
									"Fehler beim Kopieren von:\n\n");
			strlcat(mem,Pfad2,MAX_PATH);

			   ok=MessageBox(synch_hwnd,mem,0,MB_ICONERROR|MB_OKCANCEL);
			if(ok==IDCANCEL){synch_isstart=0;return -2;}
			}

		synch_isstart=0;
		return (x)? kopie_anz:-1;
		}
													// Pfad wurde nicht gefunden
	if(x==0 && y==0      ){synch_isstart=0;return -1;}
	if(x==0 && (Art&0x10)){synch_isstart=0;return -1;}

	//**************** Ganze Verzeichnisse synchronisieren ********************

			    x =CFF_FIND_ALL;
	if(Art&0x04)x|=CFF_FIND_SUBDIR;

	if(synch_name)									// Dateinamen anzeigen
		{
		   stop=synch_name(synch_param,"Durchsuche Verzeichnisse ...","");
		if(stop){synch_isstart=0;return -2;}
		}

	if(Art&0x40)
		{
		const char *temp;
		temp =Pfad2;
		Pfad2=Pfad1;
		Pfad1=temp;
		}

	if(Art&0x10)									// In beiden Richtungen
	suche2.Read(Pfad2,x,Plus,Minus);
	suche1.Read(Pfad1,x,Plus,Minus);


	len_max=0;										// Gesamt Lnge berechnen
	if(Art&0x10)
	for(x=0;x<suche2.Count();x++)len_max+=suche2.Len(x);
	for(x=0;x<suche1.Count();x++)len_max+=suche1.Len(x);

	if(len_max<=0)len_max=1;

	pos=strlcpy(puffer,Pfad2,sizeof(puffer)-16);	// Zielverzeichnis
	pos=AddBackSlash(puffer);
	suche=&suche1;

	if(synch_name)									// Dateinamen anzeigen
		{
		   stop=synch_name(synch_param,"Prfe Dateien ...","");
		if(stop){synch_isstart=0;return -2;}
		}

	for(snr=0,len_pos=0;;snr++,len_pos+=suche->Len())// Dateien kopieren
		{
		if(synch_cancel)							// Benutzer Abbruch
			{
			synch_isstart=0;
			return -2;
			}

		if(synch_percent)							// Verlauf anzeigen
			{
			x=(int)(((__int64)100*len_pos)/len_max);
			if(prozent_next<=x)
				{
				prozent_next=x/3;
				prozent_next++;
				prozent_next*=3;

				   stop=synch_percent(synch_param,x);
				if(stop)
					{
					synch_isstart=0;
					return -2;
					}
				}
			}

			name=suche->Next(1);
		if(!name)									// Wechsel der Richtung
			{
			if(~Art&0x10)break;						// Nur in eine Richtung
			if( suche==&suche2)break;
			    suche =&suche2;
				name=suche->Next(1);
			if(!name)break;

			pos=strlcpy(puffer,Pfad1,sizeof(puffer)-16);
			pos=AddBackSlash(puffer);

			if(~Art&0x01)Art|=0x08;					// Doppelkopieren verhintern
			}

		strlcpy(puffer+pos,suche->Name(-1,2),MAX_PATH-pos);

		if(Art&0x40)								// berzhlige Dateien lschen
			{
			if(ExistFile(puffer)>=1)continue;
			if(Art&0x20)							// Schreibgeschtze ignoriren
			if(suche->IsReadOnly())continue;
			}
		else{
			if(Art&0x20)							// Schreibgeschtze ignoriren
				{
				attr=GetFileAttributes(puffer);
				if(attr>0 && attr&FILE_ATTRIBUTE_READONLY)continue;
				}

			   zeit1=SynchWriteTime(puffer);		// Prfe ob Datei neuer
			if(zeit1)
				{
				if(Art&0x08)continue;				// Nie berschreiben
				if(Art&0x01 || (Art&0x10))			// Nur neuere berschreiben
					{
					   zeit2=SynchWriteTime(name);
					if(zeit2<=zeit1+1)continue;

					diff = zeit2-zeit1;
													// Sommerzeitfehler bei FAT/NTFS
					if(diff>= 1799 && diff<= 1801)continue;
					if(diff<=-1799 && diff>=-1801)continue;

					int xxx=00;
					}
				}
			}


		if(synch_name)								// Dateinamen anzeigen
			{
			synch_source=name;
			synch_target=puffer;
			   stop=synch_name(synch_param,name,puffer);
			if(stop)
				{
				synch_isstart=0;
				return -2;
				}
			}

		if(synch_percent)							// Verlauf anzeigen
			{
			x=(int)(((__int64)100*len_pos)/len_max);
			   stop=synch_percent(synch_param,x);
			if(stop)
				{
				synch_isstart=0;
				return -2;
				}
			}

		if(Art&0x80)								// Abfrage
			{
			if(Art&0x40)
				{
				strcpy (mem,"Soll die Datei:\n");
				strlcat(mem,name,MAX_PATH);
				strlcat(mem,"\ngelscht werden ?",MAX_PATH);
				}
			else{
				strcpy (mem,"Soll\n");
				strlcat(mem,name,MAX_PATH);
				strlcat(mem,"\nnach\n",MAX_PATH);
				strlcat(mem,puffer,MAX_PATH);
				strlcat(mem,"\nkopiert werden ?",MAX_PATH);
				}

			   ok=MessageBox(synch_hwnd,mem,"Frage",MB_ICONQUESTION|MB_YESNOCANCEL);
			if(ok==IDCANCEL){synch_isstart=0;return -2;}
			if(ok==IDNO)continue;
			}

		if(Art&0x40)								// berzhlige Dateien lschen
			{
			if(suche->IsDir())
					x=RemoveDirectory(name);
			else	x=RemoveFile(name);
			}
		else{
			if(suche->IsDir())						// Pfad erzeugen
					x=CreatePath(puffer);
			else	x=SynchCopyFile(name,puffer);
			}

		if(!x && (Art&0x02))						// Fehlertext ausgeben
			{
			strcpy (mem,(Art&0x40)? "Fehler beim Lschen von:\n\n":
									"Fehler beim Kopieren von:\n\n");
			strlcat(mem,name,MAX_PATH);

			   ok=MessageBox(synch_hwnd,mem,0,MB_ICONERROR|MB_OKCANCEL);
			if(ok==IDCANCEL){synch_isstart=0;return -2;}
			}

		if(!x)error=1;
		if( x)kopie_anz++;
		}

	if(synch_percent)								// Verlauf anzeigen
		{
		   stop=synch_percent(synch_param,100);
		if(stop)
			{
			synch_isstart=0;
			return -2;
			}
		}


	synch_isstart=0;


return (error)? -1:kopie_anz;
}


